#include "List.h"
#include <stdio.h>
#include <stdlib.h>

namespace ns3{

List::List(){
	len_ = 0;
	head_ = tail_ = 0;
	curr_item = 0;
}

List::~List(){
	empty();
}

void* List::next_item(){

	if(!curr_item){//start from the beginning of the list
		curr_item = head_;
	}else if(curr_item == tail_){
		return 0;
	}else{
		curr_item = curr_item->next;
	}

	if(curr_item)
		return curr_item->object;
	else
		return 0;
}


/**
 *  Insert a new packet at the end of the list
 */
void List::enqueue(void* obj){

	struct list_element* new_ele = (struct list_element*)malloc(sizeof(struct list_element));
	if(!new_ele){
		printf("Not enough memory. Terminating the process.\n");
		exit(1);
	}

	new_ele->object = obj;
	if(len_ == 0){
		head_ = tail_ = new_ele;
		new_ele->prev = new_ele->next = 0;
	}else{
		tail_->next = new_ele;
		new_ele->prev = tail_;
		new_ele->next = 0;
		tail_ = new_ele;
	}
	len_++;

	return;
}

/**
 *  Insert an element in a specific position.
 */
void List::put_position(void* obj, unsigned int pos){

	if(pos > len_ || len_==0)
	{
		enqueue(obj);

	}else if(pos>=0){
		unsigned int counter = 1;
		struct list_element* new_ele = (struct list_element*)malloc(sizeof(struct list_element));
		new_ele->object = obj;
		new_ele->prev = new_ele->next = 0;
		struct list_element* curr_ele = head_;

		while (counter < pos)
		{
			curr_ele = curr_ele->next;
			counter++;
		}

		new_ele->prev = curr_ele->prev;
		new_ele->next = curr_ele;
		if(curr_ele->prev)
			curr_ele->prev->next = new_ele;

		curr_ele->prev = new_ele;
		if(curr_ele == head_)
			head_ = new_ele;

		len_++;
	}
	
}


/**
 *  Delete all the elements in the list.
 */
void List::empty()
{
	struct list_element* curr_ele = head_;
	struct list_element* copy = head_;
	while(curr_ele)
	{
		curr_ele = copy->next;
		if (copy->object) 
			free(copy->object);
		free(copy);
		copy = curr_ele;
		len_--;
	}
	head_ = tail_ = 0;
	if(len_){
		printf("Error in List management: empty queue with non zero len. Terminating the process\n");
		exit(1);
	}
}


/**
 *  Remove from the list an element whose object is specified as parameter of the function
 */
void List::remove_item(void* obj)
{
	struct list_element* curr_ele;
	for(curr_ele = head_; curr_ele && curr_ele->object!=obj; curr_ele = curr_ele->next);
	
	if(curr_ele) //match found.
	{
		if(curr_ele->prev)
			curr_ele->prev->next = curr_ele->next;
		else
			head_ = curr_ele->next;

		if(curr_ele->next)
			curr_ele->next->prev = curr_ele->prev;
		else
			tail_ = curr_ele->prev;

		len_--;
//		if(curr_ele->object)
//			free(curr_ele->object);
		free(curr_ele);
		curr_ele = 0;
	}

}

void* List::dequeue()
{
	if(!tail_) 
		return 0;

	struct list_element *ret = tail_;
	if(tail_->prev)
		tail_->prev->next = 0;
	tail_ = tail_->prev;
	len_--;
	if(len_==0)
		head_ = tail_ = 0;

	return ret->object;
}

}//namespace