/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 TELEMATICS LAB, DEE - Politecnico di Bari
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Giuseppe Piro  <g.piro@poliba.it>
 */

#include "ip-classifier-record.h"
#include <stdint.h>
#include <iostream>
#include "ns3/ipv4-address.h"

NS_LOG_COMPONENT_DEFINE ("IpClassifierRecord");

namespace ns3 {

IpClassifierRecord::IpClassifierRecord (void)
{
  m_protocol.push_back (6); // udp
  m_protocol.push_back (17); // tcp
  AddSrcAddr (Ipv4Address ("0.0.0.0"), Ipv4Mask ("0.0.0.0"));
  AddDstAddr (Ipv4Address ("0.0.0.0"), Ipv4Mask ("0.0.0.0"));
  AddSrcPortRange (0, 65535);
  AddDstPortRange (0, 65535);
}

IpClassifierRecord::~IpClassifierRecord (void)
{
}

IpClassifierRecord::IpClassifierRecord (Ipv4Address SrcAddress,
                                        Ipv4Mask SrcMask,
                                        Ipv4Address DstAddress,
                                        Ipv4Mask DstMask,
                                        uint16_t SrcPortLow,
                                        uint16_t SrcPortHigh,
                                        uint16_t DstPortLow,
                                        uint16_t DstPortHigh,
                                        uint8_t protocol)
{
  m_protocol.push_back (protocol);
  AddSrcAddr (SrcAddress, SrcMask);
  AddDstAddr (DstAddress, DstMask);
  AddSrcPortRange (SrcPortLow, SrcPortHigh);
  AddDstPortRange (DstPortLow, DstPortHigh);
}


void
IpClassifierRecord::AddSrcAddr (Ipv4Address srcAddress, Ipv4Mask srcMask)
{
  struct ipv4Addr tmp;
  tmp.Address = srcAddress;
  tmp.Mask = srcMask;
  m_srcAddr.push_back (tmp);
}


void
IpClassifierRecord::AddDstAddr (Ipv4Address dstAddress, Ipv4Mask dstMask)
{
  struct ipv4Addr tmp;
  tmp.Address = dstAddress;
  tmp.Mask = dstMask;
  m_dstAddr.push_back (tmp);
}


void
IpClassifierRecord::AddSrcPortRange (uint16_t srcPortLow, uint16_t srcPortHigh)
{
  struct PortRange tmp;
  tmp.PortLow = srcPortLow;
  tmp.PortHigh = srcPortHigh;
  m_srcPortRange.push_back (tmp);
}


void
IpClassifierRecord::AddDstPortRange (uint16_t dstPortLow, uint16_t dstPortHigh)
{
  struct PortRange tmp;
  tmp.PortLow = dstPortLow;
  tmp.PortHigh = dstPortHigh;
  m_dstPortRange.push_back (tmp);
}


void
IpClassifierRecord::AddProtocol (uint8_t proto)
{
  m_protocol.push_back (proto);
}

}
; // namespace ns3
