/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 TELEMATICS LAB, DEE - Politecnico di Bari
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Giuseppe Piro  <g.piro@poliba.it>
 */

#include "ip-classifier.h"
#include <stdint.h>
#include "ns3/log.h"
#include "radio-bearer-instance.h"
#include "ns3/packet.h"
#include "ns3/ipv4-header.h"
#include "ns3/udp-header.h"
#include "ns3/tcp-header.h"
#include "ns3/llc-snap-header.h"
#include "ns3/udp-l4-protocol.h"
#include "ns3/tcp-l4-protocol.h"
#include "rrc-entity.h"
#include "lte-net-device.h"


NS_LOG_COMPONENT_DEFINE ("IpClassifier");

namespace ns3 {

IpClassifier::IpClassifier (void)
  : m_device (0)
{
}

IpClassifier::IpClassifier (Ptr<LteNetDevice> d)
  :   m_device (d)
{
}

IpClassifier::~IpClassifier (void)
{
}

void
IpClassifier::SetDevice (Ptr<LteNetDevice> d)
{
  m_device = d;
}


Ptr<LteNetDevice>
IpClassifier::GetDevice ()
{
  return m_device;
}


Ptr<RadioBearerInstance>
IpClassifier::Classify (Ptr<const Packet> packet)
{
  /*
   Classify is called when a packet arrives at the LTE device from
   the upper layer. It returns the Bearer in with the packet will be mapped and
   enqueued. To this aim, use a functionalities of bearer manager to find
   a proper bearer to which map the incoming packet.
   */

  // XXX: not implemented yet!
  return GetDevice ()->GetRrcEntity ()->GetDefaultBearer ();
}

}
