/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 TELEMATICS LAB, DEE - Politecnico di Bari
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Giuseppe Piro  <g.piro@poliba.it>
 */

#ifndef IP_CLASSIFIER_H_
#define IP_CLASSIFIER_H_

#include <stdint.h>
#include <vector>
#include "ns3/ptr.h"
#include "ns3/packet.h"
#include "ns3/object.h"


namespace ns3 {

class RadioBearerInstance;
class LteNetDevice;

/**
 * The IP Classifier for LTE device
 */
class IpClassifier : public Object
{
public:
  IpClassifier (void);

  /**
   * \brief create the ip classifier
   * \param d the device where the ip classifier is created
   */
  IpClassifier (Ptr<LteNetDevice> d);
  ~IpClassifier (void);

  /**
   * \brief Set the device to which the classifier is attached
   * \param d the device
   */
  void SetDevice (Ptr<LteNetDevice> d);
  /**
   * \brief Get the device to which the classifier is attached
   * \return the pointer to the device
   */
  Ptr<LteNetDevice> GetDevice ();

  /**
   * \brief classify a packet in a proper bearer
   * \param packet the packet to classify
   */
  Ptr<RadioBearerInstance> Classify (Ptr<const Packet> packet);

private:
  Ptr<LteNetDevice> m_device;

};

} // namespace ns3

#endif /* IP_CLASSIFIER_H_ */
