/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 TELEMATICS LAB, DEE - Politecnico di Bari
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Giuseppe Piro  <g.piro@poliba.it>
 */

#ifndef LTE_ERROR_MODEL_H
#define LTE_ERROR_MODEL_H


#include <ns3/spectrum-error-model.h>
#include <list>

namespace ns3 {

class PacketBurst;

/**
 * This class implements the error model for LTE based on
 * BLER curves.
 * How does the error model work?
 * When a burst of packet is received over a list of sub channels,
 * the device knows the MCS scheme used to trasmit all data. The
 * MCS scheme can be different for each sub channel.
 * The device, compute the BLER starting form MCS used for a sub
 * channel and the SINR measured for a sub channel. Than a random
 * value is chosen among [0,1]. If RAND  >= BLER the received burst
 * is erroneus.
 *
 * At this moment the error model is not implemented yet!
 */
class LteSpectrumErrorModel : public SpectrumErrorModel
{
protected:
  virtual void DoDispose ();

public:
  LteSpectrumErrorModel ();
  virtual ~LteSpectrumErrorModel ();

  // inherited from SpectrumErrorModel

  // void StartRx (Ptr<const PacketBurst> p);
  /**
   * \brief Start the RX process
   * \param p the received packet
   */
  virtual void StartRx (Ptr<const Packet> p);

  /**
   * \brief Evaluate if thers is an error at the physical layer
   * \param sinr a list of measured SINR
   * \param duration the duration of the transmission
   */
  virtual void EvaluateChunk (const SpectrumValue& sinr, Time duration);
  // void EvaluateChunk (const SpectrumValue& sinr, std::list<int> mcs);

  /**
   * \brief Check if an error has been occurred
   * \return true if there was an error
   */
  bool IsRxCorrect ();

private:
};

}


#endif /* LTE_ERROR_MODEL_H */
