/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 IMT Lucca
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Luca Anchora  <luca.anchora@imtlucca.it>
 */

#include "miscellaneous.h"

namespace ns3{

/////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////// Implementation of Class circular_queue ///////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

circular_queue::circular_queue()
{
	start_ = end_ = 0;
	num_elements_ = 0;
	tot_sum_ = 0.0;
}

void circular_queue::enqueue(double val)
{
	elements_[end_] = val;
	tot_sum_ += val;
	end_ = (end_ + 1)%BYTE_TX_LEN;
	if(num_elements_<BYTE_TX_LEN)
		num_elements_++;
	if(end_ == start_){
		tot_sum_ -= elements_[start_];
		start_ = (start_ + 1)%BYTE_TX_LEN;
	}
}

double circular_queue::sum(){
	return tot_sum_;
}

void circular_queue::reduce_last_sample(double reduction){
	if(start_ != end_){
		unsigned int last_pos = (end_+ BYTE_TX_LEN-1)%BYTE_TX_LEN;
		elements_[last_pos] -= reduction;
		tot_sum_ -= reduction;
	}
}

}; //namespace ns3
