/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2011 IMT Lucca
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Luca Anchora  <luca.anchora@imtlucca.it>
 */

#ifndef MISCELLANEOUS_H
#define MISCELLANEOUS_H
#include <ns3/mac48-address.h>
#include "List.h"

/*
 * In this file I declare and define all the data structures
 * that may be used by several other classes.
*/

namespace ns3{

class Mac48Address;
class List;

/**
 * This class implements a simple circular queue. It is used by the Scheduler to 
 * store information about the flow queues.
 */
#define BYTE_TX_LEN 100000
class circular_queue{
public:
	circular_queue();
	~circular_queue(){};
	void enqueue(double val);
	void reduce_last_sample(double reduction);
	double sum();
	unsigned int num_elements() { return num_elements_; }
private:
	double elements_[BYTE_TX_LEN];
	unsigned int start_;
	unsigned int end_;
	unsigned int num_elements_;
	double tot_sum_;
};


/**
 * This data structure contains all the information related to a queue managed by 
 * the credit-based scheduler (e.g. priority, residual credits, etc.), together 
 * with the queue itself.
 */
struct flow_queue{
	double priority;
	double credit_counter;
	List* pkt_queue;
	double tot_byte_tx;
	circular_queue* byte_tx;
	double curr_byte_tx;
	Mac48Address destUE; //MAC address of the destination UE
};


};//namespace ns3

#endif /* MISCELLANEOUS_H */