/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 TELEMATICS LAB, DEE - Politecnico di Bari
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Giuseppe Piro  <g.piro@poliba.it>
 */


#include "rrc-entity.h"
#include <ns3/log.h>
#include <ns3/pointer.h>
#include <ns3/packet.h>
#include "radio-bearer-instance.h"

NS_LOG_COMPONENT_DEFINE ("RrcEntity");

namespace ns3 {


NS_OBJECT_ENSURE_REGISTERED (RrcEntity);

TypeId RrcEntity::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::RrcEntity")
    .SetParent<Object> ()
  ;
  return tid;
}


RrcEntity::RrcEntity ()
{
  CreateBearersContainer ();
}


RrcEntity::~RrcEntity ()
{
  delete  m_downlinkGbrBearersContainer;
  delete m_downlinkNgbrBearersContainer;
  delete m_uplinkGbrBearersContainer;
  delete m_uplinkNgbrBearersContainer;
}

void
RrcEntity::CreateBearersContainer ()
{
  m_downlinkGbrBearersContainer = new BearersContainer ();
  m_downlinkNgbrBearersContainer = new BearersContainer ();
  m_uplinkGbrBearersContainer = new BearersContainer ();
  m_uplinkNgbrBearersContainer = new BearersContainer ();

  m_defaultBearer = CreateObject<RadioBearerInstance> ();
}


RrcEntity::BearersContainer*
RrcEntity::GetDownlinkGbrBearers (void) const
{
  return m_downlinkGbrBearersContainer;
}


RrcEntity::BearersContainer*
RrcEntity::GetDownlinkNgbrBearers (void) const
{
  return m_downlinkNgbrBearersContainer;
}


RrcEntity::BearersContainer*
RrcEntity::GetUplinkGbrBearers (void) const
{
  return m_uplinkGbrBearersContainer;
}


RrcEntity::BearersContainer*
RrcEntity::GetUplinkNgbrBearers (void) const
{
  return m_uplinkNgbrBearersContainer;
}


void
RrcEntity::AddBearer (Ptr<RadioBearerInstance> bearer)
{

}


Ptr<RadioBearerInstance>
RrcEntity::GetBearer (Ptr<IpClassifierRecord> ipc) const
{
  return 0;
}


Ptr<RadioBearerInstance>
RrcEntity::GetDefaultBearer (void)
{
  return m_defaultBearer;
}


} // namespace ns3
