/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2010 IMT Lucca
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Luca Anchora  <luca.anchora@imtlucca.it>
 */

#ifndef SIMPLE_PACKET_SCHEDULER_MODIFIED_H
#define SIMPLE_PACKET_SCHEDULER_MODIFIED_H

#include "simple-packet-scheduler.h"
#include "List.h"
#include "miscellaneous.h"

namespace ns3 {

/**
 * This class implements a simple credit-based packet scheduler. It also
 * implements the resource allocator: when a sequence of packets have been
 * selected for the transmission, it decides the mapping with the subchannels
 * (the resources that can be used) according to an efficiency goal. In
 * particular it tries to exploit to multiuser diversity for maximizing
 * total throughput.
 *
 * The communication between the packet scheduler and the resource allocator is
 * done through a list of candidate packets: the scheduler fills it in with a 
 * number of possible packets (Cmax), and the resource allocator selects a subset
 * of them (Creq) on the basis of the resources available and on the desired 
 * channel load.
 */

class SimplePacketSchedulerModified : public SimplePacketScheduler
{

public:
	SimplePacketSchedulerModified ();
	virtual ~SimplePacketSchedulerModified ();

	static TypeId GetTypeId (void);

	virtual void DoRunPacketScheduler (void);
	void config(Ptr<EnbNetDevice> enb); //Set the main properties for an object of this class

protected:
	virtual int determine_Cmax();//run the algorithm to obtain the value of Cmax
	virtual void fillIn_list(int Cmax);//fill in the list of candidate packets to be passed
	                                   //to the resource allocator

	void fakefillIn_list(int Cmax, struct flow_queue*);//a virtual list of candidate packets is
	                                                   //constructed to simulate an allocation

private:
	void build_optimum_list(int tot_users, int tot_subc);
	double NB_function(double D);//value of the Nash Bargaining function in a certain point
	void log_values();
	double throughput_norm();
	double fairness_norm();
	void max_throughput();

private:
	int num_ue_;//num of UE connected to the Base Station
	struct flow_queue* queues_;//array of flow queues managed by the scheduler
	double cumulative_credits_; //sum of the credits of all the flow queues
	unsigned int quantum_; //value added to the credit of each queue after each round
	int Creq_;
	int init_subframes;//number of initial fake subframes used only to send virtual packets
	
	List* attempted_list_;//temporary list built by the scheduler and used by the 
	                      //resource allocator to choose the packets to transmit
	List* optimum_list_;

	//Now a group of totalizers used at each allocation round to update the throughput
	double curr_throughput_;
	double total_throughput_;
	double curr_max_throughput_;
	double tot_max_throughput_;
	double tot_pkts_tx_;
	double curr_pkts_tx_;
	
	char not_configured;//bool parameter used for the configuration of the object
};


} //namespace

#endif /* SIMPLE_PACKET_SCHEDULER_MODIFIED_H */
